% The program calculates the spectrum of the square of the periodic real
% function g(t) using the series expansion of the self-convolution theorem
% and compares the spectrum with the FFT calculation.
clear all;
global NTS
NTS=1024;       % Number of points in the period T
NSYM=1024;      % Number of periods in the positive semiaxis. 2*NSYM = number of points in the harmonic frequency interval (1/T)
NTL=min(NSYM,2);% Number of periods shown on the negative time axis (left)
NTR=min(NSYM,2);% Number of periods shown on the positive time axis (right)
NB=30;          % Number of harminic frequency intervals shown on each frequency semiaxis
%
freq=(-NTS/2:1/(2*NSYM):(NTS-1/NSYM)/2); % Normalized frequency axis. Normalization fo=1/T
findex=(NTS*NSYM+1:NTS*NSYM+1+NB*2*NSYM);
fplot=freq(findex); % Unilateral normalized frequency axis for plotting frequrency functions
bindex=(NTS*NSYM+1-NB*2*NSYM:NTS*NSYM+1+NB*2*NSYM);
bplot=freq(bindex); % Bilateral normalized frequency axis for plotting functions
time=(-NSYM:1/NTS:NSYM-1/NTS); % Normalized time axis. Normalization T
tindex=(NTS*NSYM+1-NTL*NTS:NTS*NSYM+NTR*NTS); % Index vector of the temporal axis
tplot=time(tindex); % Normalized temporal axis used for plotting the pulse
%
%--------------------------------------------------------------------------
%
% Calculation
%
N=10; % Order of the Fourier kernel
g=sin((2*N+1)*pi*time)./(sin(pi*time));
g(1:NTS:2*NSYM*NTS)=2*N+1;
G=SPECTRUM(g);
G2=SPECTRUM(g.^2);
%
% Coefficients of the spectrum G(f) 
%
c=zeros(1,NTS+1);
for k=1:NTS
    c(k)=G(2*NSYM*(k-1)+1);
end;
c(NTS+1)=conj(c(1));
%
%---------------------- Self-convolution theorem --------------------------
%
% Coefficients of G2(f) using the self-convolution theorem
%
C=zeros(1,NTS/2);
C0=(abs(c(NTS/2+1))^2+2*sum(abs(c(NTS/2+2:NTS+1)).^2))/(2*NSYM);
C(1)=2*sum(conj(c(NTS/2+1:NTS)).*c(NTS/2+2:NTS+1))/(2*NSYM);
for k=2:NTS/2
    C(k)=(2*sum(conj(c(NTS/2+1:NTS-(k-2)-1)).*c(NTS/2+1+k:NTS+1))+...
        sum(c(NTS/2+2:NTS/2+1+k-1).*c(NTS/2+1+k-1:-1:NTS/2+2)))/(2*NSYM);
end;
G2_theorem=zeros(1,2*NTS*NSYM);
G2_theorem(NTS*NSYM+1)=C0;
for k=1:NTS/2-1
G2_theorem(NTS*NSYM+1+2*NSYM*k)=C(k);
G2_theorem(NTS*NSYM+1-2*NSYM*k)=conj(C(k));
end;
G2_theorem(1)=conj(C(NTS/2));
%
%--------------------------------------------------------------------------
%
% Graphics
%
figure(1);
subplot(211);
plot(tplot,g(tindex),'Linewidth',2);
grid on;
xlabel('Normalized time [T]');
ylabel('A.U.');
title(['Fourier kernel \kappa_N(t): N=',num2str(N,'%2d')]);
subplot(212);
semilogy (tplot,g(tindex).^2,'Linewidth',2);
grid on;
xlabel('Normalized time [T]');
ylabel('A.U.');
title(['Fourier kernel squared \kappa_N^2(t): N=',num2str(N,'%2d')]);
%
figure(2);
subplot(211);
plot(bplot,abs(G(bindex))/(2*NSYM),'Linewidth',2);
grid on;
xlabel('Normalized frequency [1/T]');
ylabel('Modulus');
title(['Spectrum G(f): N=',num2str(N)]);
subplot(212);
plot(bplot,abs(G2(bindex))/(2*NSYM),bplot,abs(G2_theorem(bindex))/(2*NSYM)+1e-100,'Linewidth',2);
grid on;
xlabel('Normalized frequency [1/T]');
ylabel('Modulus');
title(['Convolution G_2(f): N=',num2str(N)]);
